﻿Imports ThinkGeo.MapSuite
Imports ThinkGeo.MapSuite.Layers
Imports System.Collections.ObjectModel
Imports System.Drawing.Printing
Imports ThinkGeo.MapSuite.Drawing
Imports ThinkGeo.MapSuite.Shapes
Imports ThinkGeo.MapSuite.Styles
Imports ThinkGeo.MapSuite.WinForms
Imports PdfSharp.Pdf
Imports System.IO

Public Class Form1
  Private moPolyLayer As ShapeFileFeatureLayer
  Private moUsContiguousExtent As New RectangleShape(-13931965, 6319026, -7494592, 2836407)
  Public BingMapsKey As String = String.Empty
  Private moBingLayer As BingMapsLayer
  Public Enum EnumPagePosition
    TopLeft = 1
    TopRight = 2
    LowerLeft = 3
    LowerRight = 4
  End Enum
  Public Enum EnumCenterType
    Horizontal = 1
    Vertical = 2
  End Enum
  Private Sub AddCenteredTitleLabel()
    Dim oLabelPrinterLayer As LabelPrinterLayer = New LabelPrinterLayer()
    Dim oPageBoundingbox As RectangleShape

    '***** Setup the text and the font..
    With oLabelPrinterLayer
      .Text = "Page Layout"
      .Font = New GeoFont("Arial", 10, DrawingFontStyles.Bold)
      .TextBrush = New GeoSolidBrush(GeoColor.StandardColors.Yellow)

      '***** Set the labels position so that is it centered on the page one inch from the top                        
      oPageBoundingbox = GetPageBoundingBox(PrintingUnit.Inch)
      Dim oLabelCenter As PointShape = New PointShape()
      oLabelCenter.X = oPageBoundingbox.UpperRightPoint.X - oPageBoundingbox.Width / 2
      oLabelCenter.Y = oPageBoundingbox.UpperLeftPoint.Y - 10.5

      .PrinterWrapMode = PrinterWrapMode.AutoSizeText
      .ResizeMode = PrinterResizeMode.MaintainAspectRatio
      .SetPosition(5, 0.5, oLabelCenter, PrintingUnit.Inch)
    End With
    Dim oPrinterOverlay As PrinterInteractiveOverlay = mGetPrinterOverlay()
    oPrinterOverlay.PrinterLayers.Add(Guid.NewGuid.ToString, oLabelPrinterLayer)

  End Sub
  Private Sub Form1_Load(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.Load
    CreateDefaultLayout()

  End Sub
  Public Sub CreateDefaultLayout()
    Dim oExtentRectangle As RectangleShape
    Dim moPrinterOverlay As PrinterInteractiveOverlay
    Dim moPrinterPage As PagePrinterLayer

    moPrinterOverlay = New PrinterInteractiveOverlay()
    With PageLayout
      '***** Setup the map unit, you want to always use feet or meters for printer layout maps
      .MapUnit = GeographyUnit.Feet
      '***** Create the default zoom levels for the layout map
      .ZoomLevelSet = New PrinterZoomLevelSet(.MapUnit, PrinterHelper.GetPointsPerGeographyUnit(.MapUnit))

      '***** Set the background color to gray so there is contrast with the white page
      .BackgroundOverlay.BackgroundBrush = New GeoSolidBrush(GeoColor.StandardColors.LightGray)

      '***** Create the PrinterInteractiveOverlay to contain all of the PrinterLayers.
      '***** The interactive overlay allows the layers to be interacted with
      .InteractiveOverlays.Add("PrinterOverlay", moPrinterOverlay)
      .InteractiveOverlays.MoveToBottom("PrinterOverlay")

      '***** Create the PagePrinterLayer which shows the page boundary and 
      '***** is the area the user will arrange all of the other layers on top of.
      moPrinterPage = New PagePrinterLayer()
      With moPrinterPage
        .Open()
        .BackgroundMask = AreaStyles.CreateSimpleAreaStyle(GeoColor.StandardColors.White, GeoColor.SimpleColors.Red, 2)
        moPrinterOverlay.PrinterLayers.Add("PrinterPage", moPrinterPage)
        '***** Get the page's extent, scale it up by 10%
        oExtentRectangle = CType(RectangleShape.ScaleUp(.GetPosition(), 10), RectangleShape)
        '***** Set oExtentRectangle as the default current extent
      End With
      .CurrentExtent = oExtentRectangle

      '***** Set the minimum scale of the map to the last zoom level
      .MinimumScale = .ZoomLevelSet.ZoomLevel20.Scale

      '***** Add a container for the map layers
      AddMapLayerContainer(mGetPageLayoutMap)

      '***** Add a layer to the page layout
      AddMapLayers(mGetPageLayoutMap)

      AddCenteredTitleLabel()
    End With
  End Sub
  Public Sub AddMapLayers(oPageLayoutMap As MapPrinterLayer)
    Dim sPath As String
    Dim oProj4 As UnmanagedProj4Projection

    oPageLayoutMap.MapUnit = GeographyUnit.Meter

    Dim oBingLayer = mCreateBingMapsLayer()

    If oBingLayer Is Nothing Then
      Me.Close()
      Return

    End If

    sPath = "..\..\Data\SmallTracts.shp"

    '***** Projection for this shapefile is Albers (102003)
    '***** To start with we will not apply a projection to the shapefile
    moPolyLayer = New ShapeFileFeatureLayer(sPath, GeoFileReadWriteMode.Read)

    Dim oStyle = AreaStyles.CreateHatchStyle(GeoHatchStyle.ForwardDiagonal, GeoColors.DarkGray, GeoColors.Transparent, GeoColors.Black)
    'Dim oStyle = AreaStyles.CreateSimpleAreaStyle(GeoColors.Transparent, GeoColors.Yellow, 1)

    moPolyLayer.ZoomLevelSet.ZoomLevel01.DefaultAreaStyle = oStyle ' AreaStyles.CreateSimpleAreaStyle(GeoColors.Transparent, GeoColor.SimpleColors.BrightYellow, 2)
    moPolyLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20

    moPolyLayer.Open()

    Dim oFeature As Feature = moPolyLayer.QueryTools.GetFeatureById("1", ReturningColumnsType.NoColumns)

    Dim oExtent As RectangleShape = oFeature.GetBoundingBox
    oPageLayoutMap.MapExtent = moPolyLayer.GetBoundingBox

    Dim oPrinterOverlay As PrinterInteractiveOverlay = mGetPrinterOverlay()

    '***** Since the shapefile is in Srid 102003, we will need to set ProjectionToSphericalMercator
    oProj4 = New UnmanagedProj4Projection()
    oProj4.ExternalProjectionParametersString = Proj4Projection.GetDecimalDegreesParametersString
    oProj4.InternalProjectionParametersString = Proj4Projection.GetBingMapParametersString
    oProj4.Open()

    oProj4 = New UnmanagedProj4Projection()
    oProj4.ExternalProjectionParametersString = Proj4Projection.GetEsriParametersString(102003)
    oProj4.InternalProjectionParametersString = Proj4Projection.GetBingMapParametersString
    oBingLayer.ProjectionFromSphericalMercator = oProj4


    oPageLayoutMap.Layers.Add(oBingLayer)
    oPageLayoutMap.Layers.Add(moPolyLayer)
  End Sub
  Private Sub Form1_Shown(sender As Object, e As System.EventArgs) Handles Me.Shown
    Me.PageLayout.Refresh()
  End Sub
  Private Function mCreateBingMapsLayer() As BingMapsLayer
    Dim sKey As String
    If BingMapsKey = String.Empty Then
      sKey = InputBox("Please enter key for Bing Maps", "Enter BingMaps Key")
      If sKey = String.Empty Then
        Return Nothing
      End If
      BingMapsKey = sKey
    End If
    moBingLayer = New BingMapsLayer(BingMapsKey)
    moBingLayer.MapType = Layers.BingMapsMapType.Aerial
    Dim oEncryptedBitmapTileCache As New EncryptedFileBitmapTileCache("..\..\Data\BingMapsCache")
    oEncryptedBitmapTileCache.CacheId = moBingLayer.MapType.ToString
    oEncryptedBitmapTileCache.TileMatrix.TileHeight = 256
    oEncryptedBitmapTileCache.TileMatrix.TileWidth = 256

    moBingLayer.TileCache = oEncryptedBitmapTileCache
    mWriteBingMetaDataFile(moBingLayer)
    moBingLayer.ProjectedTileCache = Nothing

    Return moBingLayer
  End Function

  Public Sub SetScale(moPageLayoutMap As MapPrinterLayer)
    Dim dWidth As Single = CSng(moPageLayoutMap.GetPosition(PrintingUnit.Inch).Width * 96)
    Dim dHeight As Single = CSng(moPageLayoutMap.GetPosition(PrintingUnit.Inch).Height * 96)
    Dim oExtent As RectangleShape = ExtentHelper.ZoomToScale(15840, moPageLayoutMap.MapExtent, moPageLayoutMap.MapUnit, dWidth, dHeight)
    oExtent = ExtentHelper.CenterAt(oExtent, moPageLayoutMap.MapExtent.GetCenterPoint, dWidth, dHeight)
    moPageLayoutMap.MapExtent = oExtent
    RefreshPageLayout()
  End Sub
  Public Sub AddMapLayerContainer(moPageLayoutMap As MapPrinterLayer)
    Dim oPageBoundingbox As RectangleShape = GetPageBoundingBox(PrintingUnit.Inch)

    moPageLayoutMap = New MapPrinterLayer()
    moPageLayoutMap.MapUnit = GeographyUnit.Feet

    '***** Create the MapPrinterLayer and set the position
    moPageLayoutMap.BackgroundMask = AreaStyles.CreateSimpleAreaStyle(GeoColor.StandardColors.White, GeoColor.SimpleColors.Red, 2)

    '***** Set the maps position slightly below the pages center and 7.5 inches wide and 9 inches tall
    oPageBoundingbox = GetPageBoundingBox(PrintingUnit.Inch)
    moPageLayoutMap.SetPosition(8, 10.5, oPageBoundingbox.GetCenterPoint().X, oPageBoundingbox.GetCenterPoint().Y, PrintingUnit.Inch)

    '***** Add the MapPrinterLayer to the PrinterInteractiveOverlay
    Dim moPrinterOverlay As PrinterInteractiveOverlay = mGetPrinterOverlay()
    moPrinterOverlay.PrinterLayers.Add("MapPrinterLayer", moPageLayoutMap)
  End Sub
  Private Function GetPageBoundingBox(ByVal unit As PrintingUnit) As RectangleShape
    Dim moPrinterPage As PagePrinterLayer = mGetPrinterPage()
    Return moPrinterPage.GetPosition(unit)
  End Function
  Private Sub mWriteBingMetaDataFile(oBingLayer As BingMapsLayer)
    Dim loginServiceTemplate As String = "http://dev.virtualearth.net/REST/v1/Imagery/Metadata/{0}?&incl=ImageryProviders&o=xml&key={1}"
    Dim strUri As String = ""
    Dim wc As New Net.WebClient
    Dim xmlText As String = ""
    Dim sBingFilePath As String = CType(oBingLayer.TileCache, EncryptedFileBitmapTileCache).CacheDirectory + "\"
    Dim sMapType As String

    Select Case oBingLayer.MapType
      Case Layers.BingMapsMapType.Aerial
        sMapType = "Aerial"
      Case Layers.BingMapsMapType.AerialWithLabels
        sMapType = "AerialWithLabelsOnDemand"
      Case Else
        sMapType = "RoadOnDemand"
    End Select

    If Not Directory.Exists(sBingFilePath) Then
      Directory.CreateDirectory(sBingFilePath)
    End If

    If File.Exists(sBingFilePath & "BingMapsMetadata.xml") Then
      File.Delete(sBingFilePath & "BingMapsMetadata.xml")
    End If
    strUri = String.Format(System.Globalization.CultureInfo.InvariantCulture, loginServiceTemplate, sMapType, BingMapsKey)
    xmlText = wc.DownloadString(strUri)
    File.WriteAllText(sBingFilePath & "BingMapsMetadata.xml", xmlText)

  End Sub

  Public Sub mSetPosition(ByVal dWidth As Double, ByVal dHeight As Double, ByVal oPrinterLayer As PrinterLayer, ByVal eInitialPosition As EnumPagePosition, moPageLayoutMap As MapPrinterLayer)
    Dim dCenterPointX As Double
    Dim dCenterPointY As Double
    Dim dDistanceFromEdge As Double = 0.25

    '***** Get the bounding box for the MapPrinterLayer
    moPageLayoutMap.Open()
    Dim oMapBoundingBox As RectangleShape = moPageLayoutMap.GetPosition(PrintingUnit.Inch)
    Select Case eInitialPosition
      Case EnumPagePosition.LowerRight
        dCenterPointX = oMapBoundingBox.LowerRightPoint.X - (dWidth / 2) - dDistanceFromEdge
        dCenterPointY = oMapBoundingBox.LowerRightPoint.Y + (dHeight / 2) + dDistanceFromEdge
      Case EnumPagePosition.LowerLeft
        dCenterPointX = oMapBoundingBox.LowerLeftPoint.X + (dWidth / 2) + dDistanceFromEdge
        dCenterPointY = oMapBoundingBox.LowerLeftPoint.Y + (dHeight / 2) + dDistanceFromEdge
      Case EnumPagePosition.TopRight
        dCenterPointX = oMapBoundingBox.UpperRightPoint.X - (dWidth / 2) - dDistanceFromEdge
        dCenterPointY = oMapBoundingBox.UpperRightPoint.Y - (dHeight / 2) - dDistanceFromEdge
      Case EnumPagePosition.TopLeft
        dCenterPointX = oMapBoundingBox.UpperLeftPoint.X + (dWidth / 2) + dDistanceFromEdge
        dCenterPointY = oMapBoundingBox.UpperLeftPoint.Y - (dHeight / 2) - dDistanceFromEdge
    End Select

    '***** Now let's try to place this layer 
    oPrinterLayer.SetPosition(dWidth, dHeight, dCenterPointX, dCenterPointY, PrintingUnit.Inch)
  End Sub

  Private Sub RefreshPageLayout()
    mRefreshMapLayers(mGetPageLayoutMap)
  End Sub

  Private Sub mRefreshMapLayers(moPageLayoutMap As MapPrinterLayer)
    If Not moPageLayoutMap Is Nothing Then
      moPageLayoutMap.IsDrawing = True
    End If

    PageLayout.Refresh()

    If Not moPageLayoutMap Is Nothing Then
      moPageLayoutMap.IsDrawing = False
    End If
  End Sub
  Private Function mGetPrinterOverlay() As PrinterInteractiveOverlay
    For Each oPrinterOverlay In PageLayout.InteractiveOverlays
      If TypeOf oPrinterOverlay Is PrinterInteractiveOverlay Then
        Return CType(oPrinterOverlay, PrinterInteractiveOverlay)
      End If
    Next oPrinterOverlay
    Return Nothing
  End Function
  Private Function mGetPageLayoutMap() As MapPrinterLayer
    Dim oPrinterOverlay As PrinterInteractiveOverlay = mGetPrinterOverlay()
    Dim oPrinterLayer As PrinterLayer
    For Each oPrinterLayer In oPrinterOverlay.PrinterLayers
      If TypeOf oPrinterLayer Is MapPrinterLayer Then
        Return CType(oPrinterLayer, MapPrinterLayer)
      End If
    Next oPrinterLayer
    Return Nothing
  End Function
  Private Function mGetPrinterPage() As PagePrinterLayer
    Dim oPrinterOverlay As PrinterInteractiveOverlay = mGetPrinterOverlay()
    Dim oPrinterLayer As PrinterLayer
    For Each oPrinterLayer In oPrinterOverlay.PrinterLayers
      If TypeOf oPrinterLayer Is PagePrinterLayer Then
        Return CType(oPrinterLayer, PagePrinterLayer)
      End If
    Next oPrinterLayer
    Return Nothing
  End Function
  Public Sub Print()
    '***** Get the printer for printing
    Dim oPrinterSettings As PrinterSettings = GetPrinter()
    If oPrinterSettings Is Nothing Then
      Return
    End If

    Dim oPrinterOverLay As PrinterInteractiveOverlay = mGetPrinterOverlay()
    Dim oPagePrinterLayer As PagePrinterLayer = mGetPrinterPage()

    Dim oPrintDocument As New PrintDocument()
    With oPrintDocument
      .PrinterSettings = oPrinterSettings
      .DefaultPageSettings.Landscape = True
      If oPagePrinterLayer.Orientation = PrinterOrientation.Portrait Then
        .DefaultPageSettings.Landscape = False
      End If
      .DefaultPageSettings.PaperSize = GetPrintPreviewPaperSize(oPagePrinterLayer)
    End With

    Dim oPrinterCanvas As PrinterGeoCanvas = New PrinterGeoCanvas()

    '***** Create a rectangle to represent the drawing area for the document
    oPrinterCanvas.DrawingArea = mGetPrinterCanvasDrawingArea(oPrintDocument)
    '***** Start the drawing process
    oPrinterCanvas.BeginDrawing(oPrintDocument, oPagePrinterLayer.GetBoundingBox(), PageLayout.MapUnit)

    '***** Loop through all of the PrintingLayers in the PrinterInteractiveOverlay 
    '***** and print all of the except for the PagePrinterLayer                
    Dim labelsInAllLayers As Collection(Of SimpleCandidate) = New Collection(Of SimpleCandidate)()
    For Each oPrinterLayer As PrinterLayer In oPrinterOverLay.PrinterLayers
      oPrinterLayer.IsDrawing = True

      If Not (TypeOf oPrinterLayer Is PagePrinterLayer) Then
        oPrinterLayer.Draw(oPrinterCanvas, labelsInAllLayers)
      End If
      oPrinterLayer.IsDrawing = False
    Next oPrinterLayer
    oPrinterCanvas.EndDrawing()
  End Sub
  Private Function mGetPrinterCanvasDrawingArea(oPrintDocument As PrintDocument) As Rectangle
    Dim oRectangle As Rectangle
    Dim iOriginX As Integer
    Dim iOriginY As Integer

    '***** Here we create a System.Drawing.Rectangle to be used as the drawing
    '***** area for the PrinterGeoCanvas that will draw the map on the printer.
    '***** Notice that when the page has been set to Landscape the Width and Height
    '***** must be reversed.

    '***** Note that the dimenstions of the paper size are used and not the PrintableArea
    '***** of the printer. See more comments in the Print routine for why this is so.
    With oPrintDocument.DefaultPageSettings
      If oPrintDocument.DefaultPageSettings.Landscape Then
        iOriginX = 0
        iOriginY = 0
        oRectangle = New Rectangle(iOriginX, iOriginY, Convert.ToInt32(.PaperSize.Height), Convert.ToInt32(.PaperSize.Width))
      Else
        iOriginX = 0
        iOriginY = 0
        'Dim dPercentIncrease As Double = 4 / 96
        'Dim dIncreaseX As Double = (.PaperSize.Width * dPercentIncrease) / 2
        'Dim dIncreaseY As Double = (.PaperSize.Height * dPercentIncrease) / 2
        'oRectangle = New Rectangle(iOriginX, iOriginY, Convert.ToInt32(.PaperSize.Width + dIncreaseX), Convert.ToInt32(.PaperSize.Height + dIncreaseY))
        oRectangle = New Rectangle(iOriginX, iOriginY, Convert.ToInt32(.PaperSize.Width), Convert.ToInt32(.PaperSize.Height))
      End If
    End With

    Return oRectangle
  End Function

  Public Shared Function GetPrinter() As PrinterSettings
    Dim oPrintDialog As New PrintDialog
    Dim oPrinterSettings As PrinterSettings = Nothing

    With oPrintDialog
      If Not .ShowDialog() = DialogResult.Cancel Then
        oPrinterSettings = .PrinterSettings
      End If
    End With
    Return oPrinterSettings
  End Function
  Private Function GetPrintPreviewPaperSize(ByVal pagePrinterLayer As PagePrinterLayer) As PaperSize
    Dim printPreviewPaperSize As PaperSize = New PaperSize("AnsiA", 850, 1100)
    Select Case pagePrinterLayer.PageSize
      Case PrinterPageSize.AnsiA
        printPreviewPaperSize = New PaperSize("AnsiA", 850, 1100)
      Case PrinterPageSize.AnsiB
        printPreviewPaperSize = New PaperSize("AnsiB", 1100, 1700)
      Case PrinterPageSize.AnsiC
        printPreviewPaperSize = New PaperSize("AnsiC", 1700, 2200)
      Case PrinterPageSize.AnsiD
        printPreviewPaperSize = New PaperSize("AnsiD", 2200, 3400)
      Case PrinterPageSize.AnsiE
        printPreviewPaperSize = New PaperSize("AnsiE", 3400, 4400)
      Case PrinterPageSize.Custom
        printPreviewPaperSize = New PaperSize("Custom Size", CInt(pagePrinterLayer.CustomWidth), CInt(pagePrinterLayer.CustomHeight))
      Case Else
    End Select

    Return printPreviewPaperSize
  End Function


  Public Sub New()
    InitializeComponent()
  End Sub
  Private Function GetPdfPageSize(ByVal pageSize As PrinterPageSize) As PdfSharp.PageSize
    Dim pdfPageSize As PdfSharp.PageSize = PdfSharp.PageSize.Letter
    Select Case pageSize
      Case PrinterPageSize.AnsiA
        pdfPageSize = PdfSharp.PageSize.Letter
      Case PrinterPageSize.AnsiB
        pdfPageSize = PdfSharp.PageSize.Ledger
      Case PrinterPageSize.AnsiC
        pdfPageSize = PdfSharp.PageSize.A2
      Case PrinterPageSize.AnsiD
        pdfPageSize = PdfSharp.PageSize.A1
      Case PrinterPageSize.AnsiE
        pdfPageSize = PdfSharp.PageSize.A0
      Case PrinterPageSize.Custom
        Throw New NotSupportedException()
      Case Else
        Throw New NotSupportedException()
    End Select
    Return pdfPageSize
  End Function
  Private Sub btnRefresh_Click(sender As Object, e As EventArgs) Handles btnRefresh.Click
    RefreshPageLayout()
  End Sub
  Private Sub ExportToPdfSharp()
    ' Get the PrinterInteractiveOverlay and PagePrinterLayer as we are going to need them below
    Dim printerInteractiveOverLay As PrinterInteractiveOverlay = mGetPrinterOverlay()
    Dim pagePrinterLayer As PagePrinterLayer = mGetPrinterPage()

    Dim pdfDocument As PdfSharp.Pdf.PdfDocument = Nothing
    Dim pdfPage As PdfSharp.Pdf.PdfPage = Nothing

    Try
      ' Create the PDF documents, pages, and copnfigure them
      pdfDocument = New PdfSharp.Pdf.PdfDocument()
      pdfPage = pdfDocument.AddPage()
      If pagePrinterLayer.Orientation = PrinterOrientation.Portrait Then
        pdfPage.Orientation = PdfSharp.PageOrientation.Portrait
      Else
        pdfPage.Orientation = PdfSharp.PageOrientation.Landscape
      End If
      pdfPage.Size = GetPdfPageSize(pagePrinterLayer.PageSize)

      ' Create the PdfGeoCanvas and pass tha PdfPage into the BeginDrawing 
      Dim pdfGeoCanvas As PdfGeoCanvas = New PdfGeoCanvas()

      pdfGeoCanvas.BeginDrawing(pdfPage, pagePrinterLayer.GetBoundingBox(), PageLayout.MapUnit)
      pdfGeoCanvas.Dpi = 100

      ' Loop through all of the PrinterLayers and draw them to the canvas except for the
      ' PagePrinterLayer which is just used for the visual layout
      Dim labelsInAllLayers As Collection(Of SimpleCandidate) = New Collection(Of SimpleCandidate)()
      For Each printerLayer As PrinterLayer In printerInteractiveOverLay.PrinterLayers
        printerLayer.IsDrawing = True
        If Not (TypeOf printerLayer Is PagePrinterLayer) Then
          printerLayer.Draw(pdfGeoCanvas, labelsInAllLayers)
        End If
        printerLayer.IsDrawing = False
      Next printerLayer

      ' Finish up the drawing
      pdfGeoCanvas.EndDrawing()

      ' Dave the PDF document and launch it in the default viewer to show the user
      Dim filename As String = "PrintingResults.pdf"
      pdfDocument.Save(filename)
      Process.Start(filename)
    Catch ex As Exception
      MessageBox.Show(ex.Message & " " & ex.StackTrace, "Exception")
    Finally
      If Not pdfDocument Is Nothing Then
        pdfDocument.Dispose()
      End If
    End Try
  End Sub

  Private Sub ExportToMicrosoftPDF()

    Dim oPagePrinterLayer As PagePrinterLayer = mGetPrinterPage()
    Dim oPrintDocument As New PrintDocument()

    Dim iPageCounter As Integer = 1
    Dim sPdfFileWithoutExtension As String = String.Empty
    Dim sPdfFilename As String = String.Empty
    Dim iNumPages As Integer = 1

    Dim oPrinterSettings As New PrinterSettings
    With oPrinterSettings
      '.PrinterName = "PDF995"
      .PrinterName = "Microsoft Print To PDF"
    End With

    Try
      With oPrintDocument
        .PrinterSettings = oPrinterSettings
        .DefaultPageSettings.Landscape = True
        If oPagePrinterLayer.Orientation = PrinterOrientation.Portrait Then
          .DefaultPageSettings.Landscape = False
        End If
        .DefaultPageSettings.Landscape = False
        .DefaultPageSettings.PaperSize = GetPrintPreviewPaperSize(oPagePrinterLayer)
        .PrintController = New StandardPrintController
      End With

      '***** Loop through all of the PrintingLayers in the PrinterInteractiveOverlay 
      '***** and print all of the except for the CustomPagePrinterLayer                
      Dim labelsInAllLayers As Collection(Of SimpleCandidate) = New Collection(Of SimpleCandidate)()
      Dim sKey As String = mGetPrinterOverlay.Name

      Dim oCurPrinterOverlay = mGetPrinterOverlay()
      '***** Write a value to the pdf995 ini file so it will use that filename when creating the pdf
      Dim oPrinterCanvas As PrinterGeoCanvas = New PrinterGeoCanvas()

      '***** Create a rectangle to represent the drawing area for the document
      oPrinterCanvas.DrawingArea = mGetPrinterCanvasDrawingArea(oPrintDocument)


      '***** Start the drawing process
      oPagePrinterLayer.Open()
      Dim oDrawingRect As RectangleShape = oPagePrinterLayer.GetBoundingBox
      oPrinterCanvas.BeginDrawing(oPrintDocument, oDrawingRect, GeographyUnit.Feet)
      oPrinterCanvas.Dpi = 100
      For Each oPrinterLayer As PrinterLayer In oCurPrinterOverlay.PrinterLayers
        oPrinterLayer.IsDrawing = True

        oPrinterLayer.Open()
        oPrinterLayer.Draw(oPrinterCanvas, labelsInAllLayers)

        oPrinterLayer.IsDrawing = False
      Next oPrinterLayer
      iPageCounter += 1

      oPrinterCanvas.EndDrawing()
    Catch ex As Exception
      MessageBox.Show("Yikes!")
    End Try

  End Sub

  Private Sub btnPdfSharp_Click(sender As Object, e As EventArgs) Handles btnPdfSharp.Click
    ExportToPdfSharp()
  End Sub

  Private Sub btnPrint_Click(sender As Object, e As EventArgs) Handles btnPrint.Click
    Print()
  End Sub

  Private Sub btnExportToPdf_Click(sender As Object, e As EventArgs) Handles btnExportToPdf.Click
    ExportToMicrosoftPDF()
  End Sub
End Class
